library ieee;
use ieee.std_logic_1164.all;

entity dwnldpar is
	port(
		-- parallel port data and status pins
		ppd: in std_logic_vector(7 downto 0);
		pps: out std_logic_vector(5 downto 3);

		-- programmable oscillator
		clk: in std_logic;
		
		-- Spartan2 FPGA pins
		S2_tck: out std_logic;	-- driver to Spartan2 JTAG clock
		S2_cclk: out std_logic;	-- driver to Spartan2 config clock
		S2_progb: out std_logic;	-- driver to Spartan2 program pin
		S2_csb: out std_logic;	-- driver to Spartan2 config. chip-select
		S2_wrb: out std_logic;	-- driver to Spartan2 config. write strobe
		S2_initb: in std_logic;	-- input from Spartan2 init pin
		S2_done: in std_logic;	-- input from Spartan2 done pin
		S2_d: out std_logic_vector(7 downto 0);	-- drivers to Spartan2 data pins and 7-seg LED
		S2_m: out std_logic_vector(0 downto 0);	-- Spartan2 config mode pins
		S2_clk: out std_logic;	-- clock output to Spartan2
		S2_ppd: out std_logic_vector(7 downto 0);	-- parallel port data pins to Spartan2
		S2_pps: in std_logic_vector(5 downto 3);	-- parallel port status pins from Spartan2
		fceb: inout std_logic		-- Flash chip-enable
	);
end entity dwnldpar;


architecture arch of dwnldpar is
	constant LO: std_logic := '0';
	constant HI: std_logic := '1';
	constant SLAVE_PARALLEL_MODE: std_logic_vector(0 downto 0) := "0";
	signal cclk: std_logic;
	signal config_data: std_logic_vector(3 downto 0);
	component pullup port(O: out std_logic); end component;
	
	begin
	-- disable other chips on the XSV Board so they don't interfere
	-- during the configuration of the Spartan2 FPGA
	S2_tck	<= LO;	-- deactivate Spartan2 JTAG circuit
	fceb		<= HI when S2_done=LO else 'Z';	-- disable Flash during config.

	-- connect Spartan2 configuration pins
	S2_m		<= SLAVE_PARALLEL_MODE;	-- set Spartan2 config mode pins
	u0: pullup port map(O=>S2_progb);	-- place a pullup on the Spartan2 PROGRAM pin
	S2_progb <= LO when ppd(7)=LO else 'Z';	-- programming pulse comes from parallel port
	S2_csb <= LO when S2_done=LO else 'Z';	-- enable writing of data
	S2_wrb <= LO when S2_done=LO else 'Z';	-- during Spartan2 config. phase
	cclk <= not ppd(0);
	S2_cclk <= not cclk;
	S2_d <= (config_data & ppd(5 downto 2)) when S2_done=LO else
				 "ZZZZZZ1Z" when fceb=HI else		-- show Spartan2 config status on LED-DP
				 "ZZZZZZZZ";	-- release control of Spartan2 Flash/LED pins

	process(cclk)
	begin
		if(cclk'event and cclk=HI) then
			config_data <= ppd(5 downto 2);
		end if;
	end process;
	
	-- connect the parallel port data and status pins to the Spartan2
	S2_ppd	<= (ppd(7 downto 2) & not(ppd(1 downto 0))) when fceb=HI else "ZZZZZZZZ";
	pps 		<= S2_pps;
	
	-- send the programmable oscillator clock to the Spartan2
	S2_clk <= clk;
	
end architecture arch;

